﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.IO;
using System.Xml.Linq;
using System.Threading;
using System.Xml;
using System.Xml.XPath;
using System.Collections.Specialized;
using HttpLibrary;
using System.Reflection;
using System.Collections;
using System.Configuration;
using FileHelpers;

namespace mydigitalspace
{
    public partial class frmUpload : Form
    {
        DataTable dt = new DataTable();
        string _sid = string.Empty;
        string _cookieHeader = string.Empty;
        bool _cancelled = false;
        bool _complete = false;
        Thread th;
        HttpConnection http;

        public frmUpload()
        {
            InitializeComponent();
            dt.Columns.Add("FirstName");
            dt.Columns.Add("SurName");
            dt.Columns.Add("Email");
            dt.Columns.Add("Status");
        }

        public frmUpload(string sid, string cookieHeader)
        {
            InitializeComponent();
            _cookieHeader = cookieHeader;

            dt.Columns.Add("FirstName");
            dt.Columns.Add("SurName");
            dt.Columns.Add("Email");
            dt.Columns.Add("Status");
            _sid = sid;
        }

        private void btnBrowse_Click(object sender, EventArgs e)
        {
            using (OpenFileDialog dlg = new OpenFileDialog())
            {
                dlg.Filter = "Comma Seperated Value files (*.csv)|*.csv|Text files (*.txt)|*.txt";

                if (dlg.ShowDialog(this) == DialogResult.OK)
                {
                    FileInfo fi = new FileInfo(dlg.FileName);
                    tbFile.Text = fi.FullName;
                }
            }
            RefreshData();
        }

        private void RefreshData()
        {
            dt.Rows.Clear();

            try
            {
                FileInfo fi = new FileInfo(tbFile.Text.Trim());
                if (fi.Name.EndsWith(".csv") || fi.Name.EndsWith(".txt"))
                {
                    FileHelperEngine engine = new FileHelperEngine(typeof(DataLoader));
                    DataLoader[] res = engine.ReadFile(fi.FullName) as DataLoader[];
                    foreach (DataLoader dl in res)
                    {
                        dt.Rows.Add(dl.FirstName.Trim().Replace("\"", ""), dl.SurName.Trim(), dl.Email.Trim().Replace("\"", ""), "Waiting...");
                    }
                }
                else
                {
                    MessageBox.Show("Invalid file format!");
                }

                dataGridView1.DataSource = dt;
                FormatGrid();

                SelectCheckBox(false);
            }
            catch
            {

            }
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            try
            {
                http.Dispose();
                http = null;
            }
            catch (Exception ex)
            {
            }
            finally
            {
                this.Close();
                Application.Exit();
            }
        }

        private void btnUpload_Click(object sender, EventArgs e)
        {
            if (btnUpload.Text.Equals("Upload"))
            {
                if (GetCheckedCount() > 0)
                {
                    if (MessageBox.Show("You are about to upload the selected contacts.\nContinue?", "Confirm Upload?", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                    {
                        //begin upload process
                        _cancelled = false;
                        _complete = false;
                        btnClose.Enabled = false;
                        btnBrowse.Enabled = false;
                        btnUpload.Text = "Cancel";
                        this.Refresh();
                        timer1.Enabled = true;
                        timer1.Start();

                        th = new Thread(new ThreadStart(ProcessUpload));
                        th.IsBackground = false;
                        th.Start();
                    }
                }
                else
                {
                    MessageBox.Show("You need to select at least 1 file to upload before continuing.", "No Files Selected!", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            else
            {
                //stop upload process
                _cancelled = true;
                btnClose.Enabled = true;
                btnBrowse.Enabled = true;
                btnUpload.Text = "Upload";
            }
        }

        private void FormatGrid()
        {
            dataGridView1.Columns[1].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
            dataGridView1.Columns[2].AutoSizeMode = DataGridViewAutoSizeColumnMode.AllCells;
            dataGridView1.Columns[1].ReadOnly = true;
            dataGridView1.Columns[2].ReadOnly = true;
            dataGridView1.Columns[3].ReadOnly = true;
            dataGridView1.Columns[4].ReadOnly = true;
            dataGridView1.Refresh();
        }

        private bool UploadContact(string FirstName, string SurName, string Email)
        {
            string requestURI = String.Format("https://secure.mydigitalspacelive.com/directory/ondemand/object.asp?method=PERSON_ADD&sid={0}&firstname={1}&surname={2}&email={3}", _sid, FirstName, SurName, Email);

            //string requestURI = "https://secure.mydigitalspacelive.com/directory/ondemand/object.asp?method=PERSON_ADD&sid=148678155-k-605cc080e46585ab60ce748ad492e459&firstname=1&surname=Goroka&email=Goroka@gmail.com";

            string HTML = HTMLHelper.ReadHTMLPage(requestURI, out _cookieHeader);
            
            return (HTML.Contains("OK"));
        }

        private void ProcessUpload()
        {
            //process upload
            foreach (DataGridViewRow dgvr in dataGridView1.Rows)
            {
                if (_cancelled) //break out of loop
                {
                    break;
                }
                if (!dgvr.Cells[4].Value.Equals("Uploaded"))
                {
                    bool bchecked = (bool)((DataGridViewCheckBoxCell)dgvr.Cells[0]).Value;
                    if (bchecked) //only upload checked items
                    {
                        SetControlPropertyThreadSafe(dataGridView1, "", "Uploading...", dgvr.Index);
                        dgvr.Selected = true;

                        if (UploadContact(dgvr.Cells[1].Value.ToString(), dgvr.Cells[2].Value.ToString(),dgvr.Cells[3].Value.ToString()))
                        {
                            SetControlPropertyThreadSafe(dataGridView1, "", "Uploaded", dgvr.Index);
                        }
                        else
                        {
                            SetControlPropertyThreadSafe(dataGridView1, "", "Failed", dgvr.Index);
                        }

                        Thread.Sleep(1000); //pause thread to prevent overload, this value may need to be tweaked
                        try
                        {
                            http.Dispose();
                        }
                        catch
                        {
                        }
                    }
                }
            }
            if (_cancelled == false)
            {
                _complete = true;
            }
        }

        private delegate void SetControlPropertyThreadSafeDelegate(Control control, string propertyName, object propertyValue, int Index);

        public static void SetControlPropertyThreadSafe(Control control, string propertyName, object propertyValue, int Index)
        {
            if (control.InvokeRequired)
            {
                control.Invoke(new SetControlPropertyThreadSafeDelegate(SetControlPropertyThreadSafe), new object[] { control, propertyName, propertyValue, Index });
            }
            else
            {
                DataGridView gridView = control as DataGridView;
                gridView.Rows[Index].Cells[4].Value = propertyValue;
            }
        }

        private void frmUpload_FormClosed(object sender, FormClosedEventArgs e)
        {
            try
            {
                http.Dispose();
                http = null;
            }
            catch
            {
            }
            finally
            {
                Application.Exit();
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            if (_complete)
            {
                timer1.Stop();
                timer1.Enabled = false;

                btnClose.Enabled = true;
                btnBrowse.Enabled = true;
                btnUpload.Text = "Upload";
                mydigitalspace_attachment_loader.Properties.Settings.Default.LastModifiedDate = DateTime.Now;
                mydigitalspace_attachment_loader.Properties.Settings.Default.Save();
            }
        }

        private void btnUnSelectAll_Click(object sender, EventArgs e)
        {
            SelectCheckBox(false);
        }

        private void btnSelectAll_Click(object sender, EventArgs e)
        {
            SelectCheckBox(true);
        }

        private void SelectCheckBox(bool bcheck)
        {
            foreach (DataGridViewRow dgvr in dataGridView1.Rows)
            {
                ((DataGridViewCheckBoxCell)dgvr.Cells[0]).Value = bcheck;
            }
        }

        private void btnRefresh_Click(object sender, EventArgs e)
        {
            RefreshData();
        }

        private int GetCheckedCount()
        {
            int count = 0;
            foreach (DataGridViewRow dgvr in dataGridView1.Rows)
            {
                try
                {
                    bool bchecked = (bool)((DataGridViewCheckBoxCell)dgvr.Cells[0]).Value;
                    if (bchecked)
                    {
                        count++;
                    }
                }
                catch
                {
                }
            }

            return count;
        }
    }

    [DelimitedRecord(",")]
    [IgnoreEmptyLines(true)]
    public class DataLoader
    {
        public DataLoader()
        {
        }

        [FieldQuoted()]
        public string FirstName;

        [FieldQuoted()]
        public string SurName;

        [FieldQuoted()]
        public string Email;
    }
}
